// Licensed to Elasticsearch B.V. under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Elasticsearch B.V. licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

// Code generated from the elasticsearch-specification DO NOT EDIT.
// https://github.com/elastic/elasticsearch-specification/tree/a4f7b5a7f95dad95712a6bbce449241cbb84698d

package types

import (
	"bytes"
	"errors"
	"io"

	"strconv"

	"encoding/json"
)

// ShingleTokenFilter type.
//
// https://github.com/elastic/elasticsearch-specification/blob/a4f7b5a7f95dad95712a6bbce449241cbb84698d/specification/_types/analysis/token_filters.ts#L87-L95
type ShingleTokenFilter struct {
	FillerToken                *string `json:"filler_token,omitempty"`
	MaxShingleSize             string  `json:"max_shingle_size,omitempty"`
	MinShingleSize             string  `json:"min_shingle_size,omitempty"`
	OutputUnigrams             *bool   `json:"output_unigrams,omitempty"`
	OutputUnigramsIfNoShingles *bool   `json:"output_unigrams_if_no_shingles,omitempty"`
	TokenSeparator             *string `json:"token_separator,omitempty"`
	Type                       string  `json:"type,omitempty"`
	Version                    *string `json:"version,omitempty"`
}

func (s *ShingleTokenFilter) UnmarshalJSON(data []byte) error {

	dec := json.NewDecoder(bytes.NewReader(data))

	for {
		t, err := dec.Token()
		if err != nil {
			if errors.Is(err, io.EOF) {
				break
			}
			return err
		}

		switch t {

		case "filler_token":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.FillerToken = &o

		case "max_shingle_size":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.MaxShingleSize = o

		case "min_shingle_size":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.MinShingleSize = o

		case "output_unigrams":
			var tmp interface{}
			dec.Decode(&tmp)
			switch v := tmp.(type) {
			case string:
				value, err := strconv.ParseBool(v)
				if err != nil {
					return err
				}
				s.OutputUnigrams = &value
			case bool:
				s.OutputUnigrams = &v
			}

		case "output_unigrams_if_no_shingles":
			var tmp interface{}
			dec.Decode(&tmp)
			switch v := tmp.(type) {
			case string:
				value, err := strconv.ParseBool(v)
				if err != nil {
					return err
				}
				s.OutputUnigramsIfNoShingles = &value
			case bool:
				s.OutputUnigramsIfNoShingles = &v
			}

		case "token_separator":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.TokenSeparator = &o

		case "type":
			if err := dec.Decode(&s.Type); err != nil {
				return err
			}

		case "version":
			if err := dec.Decode(&s.Version); err != nil {
				return err
			}

		}
	}
	return nil
}

// NewShingleTokenFilter returns a ShingleTokenFilter.
func NewShingleTokenFilter() *ShingleTokenFilter {
	r := &ShingleTokenFilter{}

	r.Type = "shingle"

	return r
}
