// Licensed to Elasticsearch B.V. under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Elasticsearch B.V. licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

// Code generated from the elasticsearch-specification DO NOT EDIT.
// https://github.com/elastic/elasticsearch-specification/tree/a4f7b5a7f95dad95712a6bbce449241cbb84698d

package types

import (
	"bytes"
	"errors"
	"io"

	"encoding/json"
)

// SlackAttachment type.
//
// https://github.com/elastic/elasticsearch-specification/blob/a4f7b5a7f95dad95712a6bbce449241cbb84698d/specification/watcher/_types/Actions.ts#L101-L117
type SlackAttachment struct {
	AuthorIcon *string                `json:"author_icon,omitempty"`
	AuthorLink *string                `json:"author_link,omitempty"`
	AuthorName string                 `json:"author_name"`
	Color      *string                `json:"color,omitempty"`
	Fallback   *string                `json:"fallback,omitempty"`
	Fields     []SlackAttachmentField `json:"fields,omitempty"`
	Footer     *string                `json:"footer,omitempty"`
	FooterIcon *string                `json:"footer_icon,omitempty"`
	ImageUrl   *string                `json:"image_url,omitempty"`
	Pretext    *string                `json:"pretext,omitempty"`
	Text       *string                `json:"text,omitempty"`
	ThumbUrl   *string                `json:"thumb_url,omitempty"`
	Title      string                 `json:"title"`
	TitleLink  *string                `json:"title_link,omitempty"`
	Ts         *int64                 `json:"ts,omitempty"`
}

func (s *SlackAttachment) UnmarshalJSON(data []byte) error {

	dec := json.NewDecoder(bytes.NewReader(data))

	for {
		t, err := dec.Token()
		if err != nil {
			if errors.Is(err, io.EOF) {
				break
			}
			return err
		}

		switch t {

		case "author_icon":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.AuthorIcon = &o

		case "author_link":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.AuthorLink = &o

		case "author_name":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.AuthorName = o

		case "color":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.Color = &o

		case "fallback":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.Fallback = &o

		case "fields":
			if err := dec.Decode(&s.Fields); err != nil {
				return err
			}

		case "footer":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.Footer = &o

		case "footer_icon":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.FooterIcon = &o

		case "image_url":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.ImageUrl = &o

		case "pretext":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.Pretext = &o

		case "text":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.Text = &o

		case "thumb_url":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.ThumbUrl = &o

		case "title":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.Title = o

		case "title_link":
			var tmp json.RawMessage
			if err := dec.Decode(&tmp); err != nil {
				return err
			}
			o := string(tmp)
			s.TitleLink = &o

		case "ts":
			if err := dec.Decode(&s.Ts); err != nil {
				return err
			}

		}
	}
	return nil
}

// NewSlackAttachment returns a SlackAttachment.
func NewSlackAttachment() *SlackAttachment {
	r := &SlackAttachment{}

	return r
}
